using Godot;
using System;

public class InteractionMode : Node
{
    [Export]
    private KeyList _buildMode;

    [Signal]
    delegate void OnInteractionModeChanged(Mode oldMode, Mode newMode);

    public enum Mode
    {
        SELECT = 0,
        BUILD = 1
    }

    public void Reset()
    {
        Current = Mode.SELECT;
    }

    public Mode Current
    {
        get => _current;
        set
        {
            if (_current == value)
                return;

            var old = _current;
            _current = value;

            EmitSignal(nameof(OnInteractionModeChanged), old, _current);
        }
    }
    private Mode _current = Mode.SELECT;

    public override void _Input(InputEvent @event)
    {
        base._Input(@event);

        if (!(@event is InputEventKey keyEvent))
            return;

        if (!keyEvent.Pressed)
            return;

        var key = (KeyList)keyEvent.Scancode;

        // TODO: this should be developed into a UI stack
        if (key == KeyList.Escape)
        {
            Current = Mode.SELECT;
            return;
        }

        if (key == _buildMode)
        {
            Current = Mode.BUILD;
        }
    }
}
